@@ -54,7 +54,8 @@
   (:method ((host host))
     host)
   (:method ((hostname string))
-    (make-host :hostattrs `(:hostname (,hostname)))))
+    (make-host :hostattrs `(:hostname (,hostname))
+               :propspec (make-propspec :systems nil))))
 
 (defmethod shallow-copy-host ((host host))
   (make-instance (type-of host)
@@ -132,9 +133,20 @@ Called by properties which set up such subhosts, like CHROOT:OS-BOOTSTRAPPED."
   ;; value to have all the hostattrs it would have were PROPSPEC not to be
   ;; substituted in
   (make-instance 'unpreprocessed-host
-                 :hostattrs (hostattrs
-                             (preprocess-host (shallow-copy-host host)))
-                 :propspec propspec))
+                 ;; Drop items of prerequisite data the host usually requires,
+                 ;; as we don't need them if we're not applying its usual
+                 ;; properties.
+                 :hostattrs (remove-from-plist
+                             (hostattrs
+                              (preprocess-host (shallow-copy-host host)))
+                             :data)
+                 :propspec (make-propspec
+                            ;; Add the original PROPSPEC-SYSTEMS so that we
+                            ;; know that all the hostattrs are instantiable.
+                            :systems
+                            (union (propspec-systems propspec)
+                                   (propspec-systems (host-propspec host)))
+                            :propspec (propspec-props propspec))))
 
 (defmacro defhost (hostname (&key deploy) &body properties)
   "Define a host with hostname HOSTNAME and properties PROPERTIES.
