@@ -27,6 +27,7 @@
                           #:pathname-equal
                           #:subpathp
                           #:relative-pathname-p
+                          #:absolute-pathname-p
                           #:getenv
                           #:subdirectories
                           #:directory-files
@@ -66,6 +67,7 @@
            #:pathname-equal
            #:subpathp
            #:relative-pathname-p
+           #:absolute-pathname-p
            #:getenv
            #:subdirectories
            #:directory-files
@@ -82,6 +84,21 @@
            #:compile-file*
            #:compile-file-pathname*
 
+           ;; libc.lisp
+           #:uid_t
+           #:gid_t
+
+           #:+CLONE_NEWCGROUP+
+           #:+CLONE_NEWIPC+
+           #:+CLONE_NEWNET+
+           #:+CLONE_NEWNS+
+           #:+CLONE_NEWPID+
+           #:+CLONE_NEWTIME+
+           #:+CLONE_NEWUSER+
+           #:+CLONE_NEWUTS+
+
+           #:+NS_GET_OWNER_UID+
+
            ;; util.lisp
            #:multiple-value-mapcan
            #:lines
@@ -91,7 +108,9 @@
            #:noop
            #:symbol-named
            #:memstring=
+           #:define-simple-error
            #:plist-to-cmd-args
+           #:systemd--user
 	   #:with-local-temporary-directory
            #:pathname-file
            #:directory-contents
@@ -125,6 +144,14 @@
            #:return-exit
            #:posix-login-environment
 
+           #:define-error-retval-cfun
+
+           #:chroot
+           #:unshare
+
+           #:mapc-open-input-streams
+           #:mapc-open-output-streams
+
            ;; connection.lisp
            #:establish-connection
            #:continue-connection
@@ -149,12 +176,16 @@
            #:mktemp
            #:with-remote-current-directory
            #:run-failed
+           #:failed-cmd
+           #:failed-stdout
+           #:failed-stderr
+           #:failed-exit-code
            #:runlines
            #:test
            #:remote-exists-p
            #:remote-file-stats
            #:remote-last-reboot
-           #:remote-consfigurator-cache-pathname
+           #:remote-executable-find
            #:mountpointp
            #:delete-remote-trees
            #:empty-remote-directory
@@ -191,6 +222,7 @@
            #:get-short-hostname
            #:require-data
            #:failed-change
+           #:aborted-change
            #:assert-euid-root
            #:assert-connection-supports
            #:maybe-writefile-string
@@ -218,6 +250,7 @@
            #:define-choosing-property-combinator
            #:seqprops
            #:eseqprops
+           #:eseqprops-until
            #:with-requirements
            #:silent-seqprops
            #:unapply
@@ -227,6 +260,7 @@
 	   #:as
            #:with-flagfile
            #:with-unapply
+           #:with-homedir
 
            ;; host.lisp
            #:host
@@ -284,7 +318,6 @@
            #:get-remote-cached-prerequisite-data
            #:get-local-cached-prerequisite-data
 	   #:get-highest-local-cached-prerequisite-data
-           #:get-remote-data-cache-dir
 
            #:try-register-data-source
            #:register-data-source
@@ -302,12 +335,91 @@
 
            ;; image.lisp
            #:eval-in-grandchild
+           #:eval-in-reinvoked
            #:dump-consfigurator-in-grandchild
+           #:wrong-execution-context-for-image-dump
            #:image-dumped
            #:asdf-requirements-for-host-and-features
            #:request-asdf-requirements
            #:continue-deploy*-program))
 
+(defpackage :consfigurator.util.posix1e
+  (:use #:cl #:alexandria #:consfigurator #:cffi)
+  (:export #:acl_type_t
+           #:acl_entry_t
+           #:+ACL-USER+
+           #:+ACL-GROUP+
+           #:+ACL-TYPE-ACCESS+
+           #:+ACL-TYPE-DEFAULT+
+           #:+ACL-NEXT-ENTRY+
+           #:+ACL-FIRST-ENTRY+
+
+           #:with-acl-free
+           #:acl-get-file
+           #:acl-set-file
+           #:acl-get-entry
+           #:acl-get-tag-type
+           #:acl-get-qualifier
+           #:acl-set-qualifier
+
+           #:+CAP-CHOWN+
+           #:+CAP-DAC-OVERRIDE+
+           #:+CAP-DAC-READ-SEARCH+
+           #:+CAP-FOWNER+
+           #:+CAP-FSETID+
+           #:+CAP-KILL+
+           #:+CAP-SETGID+
+           #:+CAP-SETUID+
+
+           #:+CAP-SETPCAP+
+           #:+CAP-LINUX-IMMUTABLE+
+           #:+CAP-NET-BIND-SERVICE+
+           #:+CAP-NET-BROADCAST+
+           #:+CAP-NET-ADMIN+
+           #:+CAP-NET-RAW+
+           #:+CAP-IPC-LOCK+
+           #:+CAP-IPC-OWNER+
+           #:+CAP-SYS-MODULE+
+           #:+CAP-SYS-RAWIO+
+           #:+CAP-SYS-CHROOT+
+           #:+CAP-SYS-PTRACE+
+           #:+CAP-SYS-PACCT+
+           #:+CAP-SYS-ADMIN+
+           #:+CAP-SYS-BOOT+
+           #:+CAP-SYS-NICE+
+           #:+CAP-SYS-RESOURCE+
+           #:+CAP-SYS-TIME+
+           #:+CAP-SYS-TTY-CONFIG+
+           #:+CAP-MKNOD+
+           #:+CAP-LEASE+
+           #:+CAP-AUDIT-WRITE+
+           #:+CAP-AUDIT-CONTROL+
+           #:+CAP-SETFCAP+
+           #:+CAP-MAC-OVERRIDE+
+           #:+CAP-MAC-ADMIN+
+           #:+CAP-SYSLOG+
+           #:+CAP-WAKE-ALARM+
+           #:+CAP-BLOCK-SUSPEND+
+           #:+CAP-AUDIT-READ+
+           #:+CAP-PERFMON+
+           #:+CAP-BPF+
+           #:+CAP-CHECKPOINT-RESTORE+
+
+           #:capability-p))
+
+(defpackage :consfigurator.util.linux-namespace
+  (:use #:cl
+        #:anaphora
+        #:alexandria
+        #:consfigurator
+        #:consfigurator.util.posix1e
+        #:cffi)
+  (:export #:get-ids-offset
+           #:reduce-id-maps
+           #:shift-ids
+           #:setgroups-p
+           #:get-userns-owner))
+
 (defpackage :consfigurator.property.cmd
   (:use #:cl #:consfigurator)
   (:export #:single))
@@ -397,6 +509,7 @@
   (:local-nicknames (#:os    #:consfigurator.property.os)
                     (#:file  #:consfigurator.property.file))
   (:export #:no-services
+           #:no-services-p
            #:running
            #:restarted
            #:reloaded
@@ -411,6 +524,8 @@
                     (#:service    #:consfigurator.property.service))
   (:export #:installed
            #:installed-minimally
+           #:backports-installed
+           #:backports-installed-minimally
            #:removed
            #:reconfigured
            #:service-installed-running
@@ -436,10 +551,17 @@
            #:pinned
            #:no-pdiffs))
 
+(defpackage :consfigurator.property.package
+  (:use #:cl #:alexandria #:consfigurator)
+  (:local-nicknames (#:apt #:consfigurator.property.apt))
+  (:export #:*consfigurator-system-dependencies*
+           #:package-manager-not-found
+           #:installed))
+
 (defpackage :consfigurator.connection.sbcl
   (:use #:cl #:alexandria #:consfigurator)
-  (:local-nicknames (#:os  #:consfigurator.property.os)
-                    (#:apt #:consfigurator.property.apt)))
+  (:local-nicknames (#:os      #:consfigurator.property.os)
+                    (#:package #:consfigurator.property.package)))
 
 (defpackage :consfigurator.property.user
   (:use #:cl #:consfigurator)
@@ -459,6 +581,7 @@
   (:local-nicknames (#:service   #:consfigurator.property.service)
                     (#:apt       #:consfigurator.property.apt)
                     (#:os        #:consfigurator.property.os)
+                    (#:package   #:consfigurator.property.package)
                     (#:container #:consfigurator.property.container)
                     (#:mount     #:consfigurator.property.mount)
                     (#:file      #:consfigurator.property.file))
@@ -678,6 +801,7 @@
   (:use #:cl #:alexandria #:consfigurator)
   (:local-nicknames (#:os        #:consfigurator.property.os)
                     (#:cmd       #:consfigurator.property.cmd)
+                    (#:service   #:consfigurator.property.service)
                     (#:file      #:consfigurator.property.file)
                     (#:chroot    #:consfigurator.property.chroot)
                     (#:apt       #:consfigurator.property.apt))
@@ -782,9 +906,13 @@
            #:https-vhost))
 
 (defpackage :consfigurator.property.systemd
-  (:use #:cl #:consfigurator)
-  (:export #:started
+  (:use #:cl #:consfigurator #:anaphora)
+  (:local-nicknames (#:service      #:consfigurator.property.service))
+  (:export #:daemon-reloaded
+           #:started
            #:stopped
+           #:reloaded
+           #:restarted
            #:enabled
            #:disabled
            #:masked
@@ -795,7 +923,8 @@
   (:local-nicknames (#:cmd          #:consfigurator.property.cmd)
                     (#:file         #:consfigurator.property.file)
                     (#:apt          #:consfigurator.property.apt)
-                    (#:os           #:consfigurator.property.os))
+                    (#:os           #:consfigurator.property.os)
+                    (#:service      #:consfigurator.property.service))
   (:export #:installed
            #:service
            #:policy
@@ -807,7 +936,7 @@
            #:zone-has-service
            #:zone-masquerade
            #:zone-rich-rule
-           #:zone-direct-rule
+           #:direct-rule
            #:default-zone))
 
 (defpackage :consfigurator.property.timezone
@@ -825,18 +954,45 @@
                     (#:os           #:consfigurator.property.os))
   (:export #:has-swap-file))
 
+(defpackage :consfigurator.property.lxc
+  (:use #:cl
+        #:anaphora
+        #:alexandria
+        #:consfigurator
+        #:consfigurator.util.linux-namespace
+        #:cffi)
+  (:local-nicknames (#:file         #:consfigurator.property.file)
+                    (#:apt          #:consfigurator.property.apt)
+                    (#:os           #:consfigurator.property.os)
+                    (#:service      #:consfigurator.property.service)
+                    (#:chroot       #:consfigurator.property.chroot)
+                    (#:user         #:consfigurator.property.user)
+                    (#:systemd      #:consfigurator.property.systemd))
+  (:export #:installed
+           #:user-container-started
+           #:when-user-container-running
+           #:user-containers-autostart
+           #:usernet-usable-by
+           #:user-container-for
+           #:user-container-for.
+           #:user-container
+           #:user-container.
+
+           #:lxc-ls))
+
 (defpackage :consfigurator.connection.local
   (:use #:cl #:consfigurator #:alexandria)
   (:export #:local-connection))
 
 (defpackage :consfigurator.connection.shell-wrap
-  (:use #:cl #:consfigurator)
+  (:use #:cl #:alexandria #:consfigurator)
   (:export #:shell-wrap-connection #:connection-shell-wrap))
 
 (defpackage :consfigurator.connection.fork
   (:use #:cl #:alexandria #:consfigurator #:consfigurator.connection.local)
   (:export #:fork-connection
-           #:post-fork))
+           #:post-fork
+           #:init-hooks-connection))
 
 (defpackage :consfigurator.connection.rehome
   (:use #:cl #:consfigurator #:consfigurator.connection.fork)
@@ -889,6 +1045,17 @@
   (:local-nicknames (#:re   #:cl-ppcre)
 		    (#:user #:consfigurator.property.user)))
 
+(defpackage :consfigurator.connection.linux-namespace
+  (:use #:cl
+        #:anaphora
+        #:alexandria
+        #:consfigurator
+        #:consfigurator.util.linux-namespace
+        #:consfigurator.connection.fork
+        #:consfigurator.connection.shell-wrap)
+  (:local-nicknames (#:user #:consfigurator.property.user)
+                    (#:lxc  #:consfigurator.property.lxc)))
+
 (defpackage :consfigurator.data.asdf
   (:use #:cl #:alexandria #:consfigurator))
 
