@@ -149,9 +149,9 @@ supported."
            (nreversef ,argsym)
            ,@forms)))))
 
-(defmacro define-simple-error (name &optional docstring)
+(defmacro define-simple-error (name &optional parent-types docstring)
   `(progn
-     (define-condition ,name (simple-error) ()
+     (define-condition ,name (,@parent-types simple-error) ()
        ,@(and docstring `((:documentation ,docstring))))
      (defun ,name (message &rest args)
        ,@(and docstring `(,docstring))
@@ -171,13 +171,37 @@ supported."
   (doplist (k v plist args)
            (push (strcat "--" (string-downcase (symbol-name k)) "=" v) args)))
 
+(defun systemd--user (&rest args)
+  "Where ARGS are args to RUN or MRUN for an invocation of a systemd command
+which can take \"--user\", insert the \"--user\" parameter, and modify or
+insert an :ENV parameter so that the call is more likely to succeed."
+  (loop with xrd = (format nil "/run/user/~D" (get-connattr :remote-uid))
+        with dsba = (format nil "unix:path=~A/bus" xrd)
+        with arg-done and env-done while args
+        as next = (pop args) collect next into accum
+        if (and (not arg-done) (stringp next))
+          collect "--user" into accum and do (setq arg-done t)
+        if (eql :env next)
+          collect (aprog1 (copy-list (pop args))
+                    (setf (getf it :XDG_RUNTIME_DIR) xrd
+                          (getf it :DBUS_SESSION_BUS_ADDRESS) dsba
+                          env-done t))
+            into accum
+        if (eql :input next)
+          collect (pop args)
+        finally (return (if env-done
+                            accum
+                            (list* :env `(:XDG_RUNTIME_DIR ,xrd
+                                          :DBUS_SESSION_BUS_ADDRESS ,dsba)
+                                   accum)))))
+
 (defmacro with-local-temporary-directory ((dir) &body forms)
   "Execute FORMS with a local temporary directory's pathname in DIR.
 Currently assumes GNU mktemp(1).
 
-There is no WITH-REMOTE-TEMPORARY-DIRECTORY because POSIX doesn't have a way
-to create temporary directories.  If you need a remote temporary directory,
-one solution is to convert your property to a :LISP property."
+There is no WITH-REMOTE-TEMPORARY-DIRECTORY because POSIX doesn't include a
+shell utility to create temporary directories.  If you need a remote temporary
+directory, one solution is to convert your property to a :LISP property."
   `(let ((,dir (ensure-directory-pathname
 		(stripln
 		 (run-program "umask 077; mktemp -d" :output :string)))))
@@ -556,46 +580,90 @@ interactive debugger."))
                                      (lambda (c)
                                        (trivial-backtrace:print-backtrace
                                         c :output *error-output*)
-                                       (uiop:quit 3)))
+                                       (uiop:quit 1)))
                                    (skipped-properties (lambda (c)
                                                          (declare (ignore c))
                                                          (setq ,failures t))))
                       ,@forms)))
        (uiop:quit (cond ((eql :no-change result) 0)
-                        (,failures               2)
-                        (t                       1))))))
+                        (,failures               22)
+                        (t                       23))))))
 
 (defmacro return-exit (exit &key on-failure)
   `(values
     nil
     (case ,exit
-      (0                              :no-change)
-      (1                              nil)
-      (2 (signal 'skipped-properties) nil)
-      (t                              ,on-failure))))
+      (0                               :no-change)
+      (22 (signal 'skipped-properties) nil)
+      (23                              nil)
+      (t                               ,on-failure))))
 
-(defun posix-login-environment (logname home)
+(defun posix-login-environment (&optional uid logname home)
   "Reset the environment after switching UID, or similar, in a :LISP connection.
 Does not currently establish a PAM session."
-  (let ((rootp (zerop (nix:geteuid)))
+  (let ((rootp (zerop (or uid (nix:geteuid))))
         (maybe-preserve '("TERM")))
     (when rootp
       (push "SSH_AUTH_SOCK" maybe-preserve))
     (let ((preserved (loop for var in maybe-preserve
                            for val = (getenv var)
                            when val collect var and collect val)))
-      (unless (zerop (foreign-funcall "clearenv" :int))
-        (failed-change "clearenv(3) failed!"))
+      (clearenv)
       (loop for (var val) on preserved by #'cddr do (setf (getenv var) val)))
-    (setf (getenv "HOME") (drop-trailing-slash (unix-namestring home))
-          (getenv "USER") logname
-          (getenv "LOGNAME") logname
-          (getenv "SHELL") "/bin/sh"
+    (when logname
+      (setf (getenv "USER") logname (getenv "LOGNAME") logname))
+    (when home
+      (setf (getenv "HOME") (drop-trailing-slash (unix-namestring home)))
+      (uiop:chdir home))
+    (setf (getenv "SHELL") "/bin/sh"
           (getenv "PATH")
           (if rootp
               "/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin"
-              "/usr/local/bin:/bin:/usr/bin"))
-    (uiop:chdir home)))
+              "/usr/local/bin:/bin:/usr/bin"))))
+
+
+;;;; System and libc calls which can fail
+
+;;; Osicat has an implementation of this but it's not exported.  However, we
+;;; are able to instantiate Osicat's POSIX-ERROR to simplify errno handling.
+
+(defmacro define-error-retval-cfun
+    ((&key (errno t) (failure-val -1)) &body defcfun-args)
+  (let ((defun (etypecase (car defcfun-args)
+                 (string
+                  (translate-name-from-foreign (car defcfun-args) '*package*))
+                 (list (cadar defcfun-args))))
+        (cfun (etypecase (car defcfun-args)
+                (string (car defcfun-args))
+                (list (caar defcfun-args))))
+        (failure-val-check
+          (once-only (failure-val)
+            `(cond ((numberp ,failure-val) (= ,failure-val result))
+                   ((pointerp ,failure-val) (pointer-eq ,failure-val result))
+                   (t (simple-program-error
+                       "Don't know how to compare function return value with ~S."
+                       ,failure-val))))))
+    `(defun ,defun ,(loop for arg in (cddr defcfun-args) collect (car arg))
+       ,@(and (eql errno :zero) '((nix:set-errno 0)))
+       (let ((result (foreign-funcall
+                      ,cfun
+                      ,@(loop for arg in (cddr defcfun-args)
+                              collect (cadr arg) collect (car arg))
+                      ,(cadr defcfun-args))))
+         (if ,(if (eql errno :zero)
+                  `(and ,failure-val-check (not (zerop (nix:get-errno))))
+                  failure-val-check)
+             (nix:posix-error ,(and errno '(nix:get-errno)) nil ',defun)
+             result)))))
+
+
+;;;; Miscellaneous system functions
+
+(define-error-retval-cfun () "clearenv" :int)
+
+(define-error-retval-cfun () "chroot" :int (path :string))
+
+(define-error-retval-cfun () "unshare" :int (flags :int))
 
 
 ;;;; Lisp data files
